/*
 * Created on 21.04.2004
 *
 * (C) Peter Weigerber <peter.weissgerber@ku-eichstaett.de>
 * and Michael Burch <michael.burch@ku-eichstaett.de>
 * 
 * For license information see the LICENSE file that is shipped
 * with the source code.
 * 
 * 
 */
package org.eposoft.epos.eposee.pixmap.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

/** This class implements the color scale selection dialog.
 * 
 * @author Peter Weigerber <peter.weigerber@ku-eichstaett.de>
 * @author Michael Burch <michael.burch@ku-eichstaett.de> 
 * 
 */
public class ColorScaleDialog extends Dialog {
	
	/** Create new color scale selection dialog.
	 * 
	 * @param arg0 shell of the dialog
	 */
	public ColorScaleDialog(Shell arg0) {
		super(arg0);
	}
	
	private Button btnOkay;
	private Button btnCancel;
	private RGB[] selection = null;
	private ColorScaleCanvas cscale[];
	private Button csbutton[];

	/** Open the dialog and let the user pick a scale.
	 * 
	 * @return selected color scale as array of RGB colors
	 */
	public RGB[] open() {
		final Shell shell =
			new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("Select Appropriate Color Scale");
		shell.setSize(400, 500);
	
		//lblQuestion = new Label(shell, SWT.NONE);
		//lblQuestion.setLocation(25, 10);
		//lblQuestion.setSize(150, 20);
		//lblQuestion.setText("What is your name?");
		cscale = new ColorScaleCanvas[SWTColorScales.colorscales.length];
		csbutton = new Button[SWTColorScales.colorscales.length];
		for (int i = 0; i < SWTColorScales.colorscales.length; i++) {
			csbutton[i] = new Button(shell, SWT.RADIO);
			csbutton[i].setLocation(10, 20 + i*40);
			csbutton[i].setSize(20,20);
			csbutton[i].setToolTipText("Select color scale \""+SWTColorScales.scalenames[i]+"\"");
			csbutton[i].addListener(SWT.Selection, new ButtonActivatedListener());
			cscale[i] = new ColorScaleCanvas(shell, SWT.NONE, SWTColorScales.colorscales[i], SWTColorScales.scalenames[i]);
			cscale[i].setLocation(40,10+i*40);
			cscale[i].setSize(350,40);
		}
		
		csbutton[0].setSelection(true);
		cscale[0].setActivated(true);
		
		btnOkay = new Button(shell, SWT.PUSH);
		btnOkay.setText("Ok");
		btnOkay.setLocation(40, 426);
		btnOkay.setSize(100, 35);
		btnCancel = new Button(shell, SWT.PUSH);
		btnCancel.setText("Cancel");
		btnCancel.setLocation(260, 426);
		btnCancel.setSize(100, 35);
		Listener listener = new Listener() {
			public void handleEvent(Event event) {
				if (event.widget == btnOkay) {
					boolean found = false;
					for (int i=0;(i<SWTColorScales.colorscales.length) && (!found);i++) {
						if (csbutton[i].getSelection()) {
							selection = cscale[i].getSelectedColors();
							found = true;
						}
					}
				}
				else {
					selection = null;
				}
				shell.close();
			}
		};
		btnOkay.addListener(SWT.Selection, listener);
		btnCancel.addListener(SWT.Selection, listener);
		shell.open();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		return selection;
	}

	public class ButtonActivatedListener implements Listener {

		/* (non-Javadoc)
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event e) {
			for (int i = 0; i < SWTColorScales.colorscales.length; i++) {
				cscale[i].setActivated(e.widget == csbutton[i]);
			}
		}
		
	}

}
