package org.eposoft.epospix.rose.actions;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.lang.reflect.Array;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.team.internal.ccvs.core.CVSException;
import org.eclipse.team.internal.ccvs.core.resources.CVSWorkspaceRoot;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eposoft.epos.eposee.pixmap.data.SparePixMap;
import org.eposoft.epos.eposee.pixmap.gui.EPOSpix;
import org.softevo.rose.assist.AssistPlugin;
import org.softevo.rose.data.entities.db.DatabaseOperations;
import org.softevo.rose.data.entities.db.Tables;
import org.softevo.rose.db.DatabaseProvider;
import org.softevo.rose.db.DbPlugin;

/**
 * Our sample action implements workbench action delegate.
 * The action proxy will be created by the workbench and
 * shown in the UI. When the user tries to use the action,
 * this delegate will be created and execution will be 
 * delegated to it.
 * @see IWorkbenchWindowActionDelegate
 */
public class EposeePixelmapAction implements IObjectActionDelegate {
	private IWorkbenchWindow window;
	/**
	 * The constructor.
	 */
	public EposeePixelmapAction() {
	}

    /**
     * Find the object associated with the given object when it is adapted to
     * the provided class. Null is returned if the given object does not adapt
     * to the given class
     * 
     * @param selection
     * @param c
     * @return Object
     */
    public static Object getAdapter(Object adaptable, Class c) {
        if (c.isInstance(adaptable)) {
            return adaptable;
        }
        if (adaptable instanceof IAdaptable) {
            IAdaptable a = (IAdaptable) adaptable;
            Object adapter = a.getAdapter(c);
            if (c.isInstance(adapter)) {
                return adapter;
            }
        }
        return null;
    }
    
    /**
     * Creates an array of the given class type containing all the
     * objects in the selection that adapt to the given class.
     * 
     * @param selection
     * @param c
     * @return
     */
    public static Object[] getSelectedAdaptables(ISelection selection, Class c) {
        ArrayList result = null;
        if (!selection.isEmpty()) {
            result = new ArrayList();
            Iterator elements = ((IStructuredSelection) selection).iterator();
            while (elements.hasNext()) {
                Object adapter = getAdapter(elements.next(), c);
                if (c.isInstance(adapter)) {
                    result.add(adapter);
                }
            }
        }
        if (result != null && !result.isEmpty()) {
            return result.toArray((Object[])Array.newInstance(c, result.size()));
        }
        return (Object[])Array.newInstance(c, 0);
    }

    
    private IStructuredSelection selection;
    private Shell shell;
    private IWorkbenchPart targetPart;

    /** Runs the action and pops up the dialog. <BR>
     */
    public void run(IAction action) {
        IProject[] selectedProjects = getSelectedProjects();

        IProject project = selectedProjects[0];
        DatabaseProvider db = DbPlugin.getDefault().getProviderForProject(project);
        DatabaseOperations dbOperations = new DatabaseOperations(db);
        
        String lineitemTable = Tables.TABLE_REVISIONS;
        String fileTable = Tables.TABLE_FILES;
        try {
            dbOperations.init();

            ArrayList files = new ArrayList();
            TreeMap map = new TreeMap();
            int id = 0;
            String query = "SELECT FileId, QualifiedFileName FROM " + fileTable + " ORDER BY Directoryname, FileName";
            ResultSet rset = dbOperations.executeQuery(query);
            while (rset.next()) {
                map.put(new Integer(rset.getInt(1)), new Integer(id));
                files.add(rset.getString(2));
                id++;
            }
            rset.close();
            SparePixMap mypix = new SparePixMap(id);
            String[] mappings = (String[]) files.toArray(new String[0]);
            mypix.setMappings(mappings);
            
            query = 
                "SELECT "
                    + "r.FileIdL, r.FileIdR, "
                    + "r.Count1 AS SupportCount, "
                    //+ "c.Count2 AS SupportCountL, "
                    //+ "r.Count1/t.Count3 AS Support, "
                    + "r.Count1*1.0/c.Count2 AS Confidence "

                //+ "INTO " + resultTable + " "
                + "FROM "

                    + "(SELECT "
                        + "l.FileId AS FileIdL, m.FileId AS FileIdR, "
                        + "COUNT(*) AS Count1 "
                        + "FROM " 
                            + lineitemTable + " l, "
                            + lineitemTable + " m "
                        + "WHERE l.TransaktionId=m.TransaktionId "
                        //+ "AND l.RevisionID<>'1.1' AND l.State<>'dead' "
                        //+ "AND m.RevisionID<>'1.1' AND m.State<>'dead' "
                        //+ "AND l.FileId<>m.FileId "
                        + "GROUP BY l.FileId, m.FileId) r, "
                
                    + "(SELECT FileId, COUNT(*) As Count2 "
                        + "FROM " + lineitemTable + " "
                        //+ "WHERE RevisionID<>'1.1' AND State<>'dead' "
                        + "GROUP BY FileId) c "
                
                    //+ "(SELECT COUNT(*)*1.0 AS Count3 FROM " +
                    //    "(SELECT DISTINCT TransaktionId FROM " + lineitemTable + ") foo) t "

                + "WHERE r.FileIdL=c.FileId;";        
            rset = dbOperations.executeQuery(query);
            while (rset.next()) {
                int itemx = ((Integer) map.get(new Integer(rset.getInt(1)))).intValue();
                int itemy = ((Integer) map.get(new Integer(rset.getInt(2)))).intValue();
                int support = rset.getInt(3);
                double confidence = rset.getDouble(4);
                mypix.putCoupling(itemx, itemy, support, confidence);            
            }
            rset.close();
            
            new EPOSpix(shell.getDisplay(), mypix);            
            
        } catch (SQLException e) {
            AssistPlugin.logException(e, Status.ERROR);
        }
    }
    
    /**
     * Returns the selected projects.
     * 
     * @return the selected projects
     */
    protected IProject[] getSelectedProjects() {
        IResource[] selectedResources = getSelectedResources();
        if (selectedResources.length == 0) return new IProject[0];
        ArrayList projects = new ArrayList();
        for (int i = 0; i < selectedResources.length; i++) {
            IResource resource = selectedResources[i];
            if (resource.getType() == IResource.PROJECT) {
                projects.add(resource);
            }
        }
        return (IProject[]) projects.toArray(new IProject[projects.size()]);
    }
    
    /**
     * Returns the selected resources.
     * 
     * @return the selected resources
     */
    protected IResource[] getSelectedResources() {
        return (IResource[]) getSelectedResources(IResource.class);
    }
    
    /**
     * Returns an array of the given class type c that contains all
     * instances of c that are either contained in the selection or
     * are adapted from objects contained in the selection.
     * 
     * @param c
     * @return
     */
    protected Object[] getSelectedResources(Class c) {
        return getSelectedAdaptables(selection, c);
    }

    /*
     * Method declared on IActionDelegate.
     */
    public void selectionChanged(IAction action, ISelection selection) {
        if (selection instanceof IStructuredSelection) {
            this.selection = (IStructuredSelection) selection;
            if (action != null) {
                setActionEnablement(action);
            }
        }
    }
    
    /**
     * Method invoked from <code>selectionChanged(IAction, ISelection)</code> 
     * to set the enablement status of the action. The instance variable 
     * <code>selection</code> will contain the latest selection so the methods
     * <code>getSelectedResources()</code> and <code>getSelectedProjects()</code>
     * will provide the proper objects.
     * 
     * This method can be overridden by subclasses but should not be invoked by them.
     */
    protected void setActionEnablement(IAction action) {
        action.setEnabled(isEnabled());
    }
    
    /**
     * @return
     */
    private boolean isEnabled() {
        if (AssistPlugin.getDefault().isPreprocessing()) return false;
        
        IProject[] projects = getSelectedProjects();
        if (projects.length<1) return false;

        for (int i = 0; i < projects.length; i++) {
            try {
                if (!CVSWorkspaceRoot.isSharedWithCVS(projects[i])) return false;
                if (DbPlugin.getDefault().getProviderForProject(projects[i])==null) return false;
            } catch (CVSException e) {
                AssistPlugin.logException(e, Status.WARNING);
                return false;
            }
        }
        
        return true;
    }

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
	}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}

    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        if(targetPart != null) {
        this.shell = targetPart.getSite().getShell();
        this.targetPart = targetPart;
    }
    }
}