/*
 * $Id: ChangedEntitiesView.java,v 1.11 2005/01/13 14:50:17 dallmeier Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.assist.ui.views;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.views.tasklist.ITaskListResourceAdapter;
import org.softevo.rose.assist.AssistPlugin;
import org.softevo.rose.assist.IChangedEntitiesListener;
import org.softevo.rose.compare.ModificationItem;
import org.softevo.rose.db.DbPlugin;



/** This view presents the list of entities that
 *  where changed during the eclipse session. <BR>
 *  It registers itself in the <code>AssistPlugin</code>
 *  class and listens for changes in the list of changed
 *  entities.
 */
public class ChangedEntitiesView extends ViewPart implements IChangedEntitiesListener {
  
  /** Clears the list of changed entities. <BR>
   */
  private Action clearAction;
  
  /** The viewer for the item tree. <BR>
   */
  private TreeViewer treeViewer = null;
  
  
  class NameSorter extends ViewerSorter { /*empty*/  }
  
  private ISelectionListener focusListener = new ISelectionListener() {
      public void selectionChanged(IWorkbenchPart part, ISelection selection) {
          ChangedEntitiesView.this.focusSelectionChanged(part, selection);
      }
  };
  
  private HashSet focussedProjects = new HashSet();
  
  /**
   * The constructor.
   */
  public ChangedEntitiesView() {
    AssistPlugin.getDefault().addChangedEntitiesListener(this);
  }
  
  /**
   * This is a callback that will allow us
   * to create the viewer and initialize it.
   */
  public void createPartControl(Composite parent) {

      getSite().getPage().addSelectionListener(focusListener);
      focusSelectionChanged(getSite().getPage().getActivePart(), getSite().getPage().getSelection());
  
    treeViewer = new TreeViewer(parent);
    treeViewer.setContentProvider(new ITreeContentProvider() {
        public Object[] getChildren(Object parentElement) {
            return null;
        }
        public Object getParent(Object element) {
            return null;
        }
        public boolean hasChildren(Object element) {
            return false;
        }
        public Object[] getElements(Object inputElement) {
            return (Object[]) treeViewer.getInput();
        }
        public void dispose() { /* empty */ }

        public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
            // TODO Auto-generated method stub

        }
    });
    treeViewer.setLabelProvider(new ILabelProvider() {
        public Image getImage(Object element) {
            if (!(element instanceof ModificationItem)) return null;
            ModificationItem item = (ModificationItem) element;
            switch (item.getType()) {
            case ModificationItem.ADDED:
                return AssistPlugin.getDefault().getImageRegistry().getDescriptor("plus").createImage();
            case ModificationItem.CHANGED:
                return AssistPlugin.getDefault().getImageRegistry().getDescriptor("change").createImage();
            case ModificationItem.DELETED:
                return AssistPlugin.getDefault().getImageRegistry().getDescriptor("minus").createImage();
            default:
            }
            return null;        
        }

        public String getText(Object element) {
            if (element instanceof ModificationItem) {
                return ((ModificationItem) element).getID();
            }
            return null;
        }

        public void addListener(ILabelProviderListener listener) { /* empty */ }
        public void dispose() { /* empty */ }
        public boolean isLabelProperty(Object element, String property) { return false; }
        public void removeListener(ILabelProviderListener listener) { /* empty */ }
    });
    treeViewer.setSorter(new NameSorter());
    treeViewer.setInput(getRelevantModificationItems());
    makeActions();
    hookContextMenu();
    hookDoubleClickAction();
    contributeToActionBars();
  }
  
  /**
   * @param activePart
   * @param selection
   */
  private void focusSelectionChanged(IWorkbenchPart activePart, ISelection selection) {
      List resources = new ArrayList();
      if (activePart instanceof IEditorPart) {
          IEditorInput input = ((IEditorPart) activePart).getEditorInput();
          if (input instanceof FileEditorInput) {
              resources.add(((FileEditorInput) input).getFile());
          }
      } else {
          if (selection instanceof IStructuredSelection) {
              for (Iterator iterator = ((IStructuredSelection) selection).iterator(); iterator.hasNext();) {
                  Object object = iterator.next();
                  if (object instanceof IAdaptable) {
                      ITaskListResourceAdapter taskListResourceAdapter;
                      Object adapter = ((IAdaptable) object).getAdapter(ITaskListResourceAdapter.class);
                      IResource resource = null;
                      if (adapter != null && adapter instanceof ITaskListResourceAdapter) {
                          taskListResourceAdapter = (ITaskListResourceAdapter) adapter;
                          resource = taskListResourceAdapter.getAffectedResource((IAdaptable) object);
                      } else {
                          IAdaptable adaptable = (IAdaptable) object;
                          resource = (IResource) adaptable.getAdapter(IResource.class);
                          if (resource == null)
                              resource = (IFile) adaptable.getAdapter(IFile.class);
                      }

                      if (resource != null) {
                          resources.add(resource);
                      }
                  }
              }
          }
      }

      boolean updateFocussedProjects = false;
      HashSet projects = new HashSet();
      for (Iterator iter = resources.iterator(); iter.hasNext();) {
          IProject project = ((IResource) iter.next()).getProject();
          projects.add(project);
          updateFocussedProjects = updateFocussedProjects || !focussedProjects.contains(project);
      }
      
      if (updateFocussedProjects) {
          focussedProjects = new HashSet();
          for (Iterator iter = projects.iterator(); iter.hasNext();) {
              IProject project = (IProject) iter.next();
              focussedProjects.addAll(Arrays.asList(DbPlugin.getDefault().getColocatedProjects(project)));
          }
          recalculate();
      }    
  }

  
/** Called to refresh the view. <BR>
 */
private void recalculate() {
    if (treeViewer==null) return;
    treeViewer.setInput(getRelevantModificationItems());
    treeViewer.refresh();
}

private void hookContextMenu() {
    MenuManager menuMgr = new MenuManager("#PopupMenu");
    menuMgr.setRemoveAllWhenShown(true);
    menuMgr.addMenuListener(new IMenuListener() {
      public void menuAboutToShow(IMenuManager manager) {
        ChangedEntitiesView.this.fillContextMenu(manager);
      }
    });
    Menu menu = menuMgr.createContextMenu(treeViewer.getControl());
    treeViewer.getControl().setMenu(menu);
    getSite().registerContextMenu(menuMgr, treeViewer);
  }
  
  private void contributeToActionBars() {
    IActionBars bars = getViewSite().getActionBars();
    fillLocalPullDown(bars.getMenuManager());
    fillLocalToolBar(bars.getToolBarManager());
  }
  
  /** Adds the clear action to the pulldown menu. <BR>
   */
  private void fillLocalPullDown(IMenuManager manager) {
    manager.add(clearAction);
    manager.add(new Separator());
  }
  
  private void fillContextMenu(IMenuManager manager) {
    manager.add(clearAction);
    // Other plug-ins can contribute there actions here
    manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
  }
  
  private void fillLocalToolBar(IToolBarManager manager) {
    manager.add(clearAction);
  }
  
  private void makeActions() {
    clearAction = new Action() {
      public void run() {
        AssistPlugin.getDefault().clearChangedEntitiesList();
      }
    };
    clearAction.setText("Clear");
    clearAction.setToolTipText("Clears the list of changed entities");
    clearAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
                               getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
  }

  private void openSelection(ISelection selection) {
      if (selection instanceof IStructuredSelection) {
          Object o = ((IStructuredSelection) selection).getFirstElement();
          if (o instanceof ModificationItem) {
              ModificationItem mi = (ModificationItem) o;
              if (mi.getAttachedFileName() != null && mi.getType() != ModificationItem.DELETED) {
                  IProject[] projects = new IProject[focussedProjects.size()];
                  int i = 0;
                  for (Iterator iter = focussedProjects.iterator(); iter.hasNext(); i++) {
                    projects[i] = (IProject) iter.next();
                  }
                  AssistPlugin.getDefault().openInEditor(mi.getAttachedFileName(), mi.getID(), projects);
              }
          } 
      }
  }

  
  private void hookDoubleClickAction() {
    treeViewer.addDoubleClickListener(new IDoubleClickListener() {
      public void doubleClick(DoubleClickEvent event) {
          openSelection(event.getSelection());
      }
    });
  }
  

  /** Refreshes the tree view. <BR>
   *  @param changedEntities the list of changed entities
   */
  public void updateChangedEntities(HashMap changedEntities, Vector changedProjects) {
      // Lazy update
      if (changedProjects==null) {
          recalculate();
          return;          
      }
      for (Iterator iter = changedProjects.iterator(); iter.hasNext();) {
        Object element = iter.next();
        if (focussedProjects.contains(element)) {
            recalculate();
            return;
        }
      }
  }
  
  /**
   * Passing the focus request to the viewer's control.
   */
  public void setFocus() {
    treeViewer.getControl().setFocus();
  }
  
    private ModificationItem[] getRelevantModificationItems() {
        Vector union = new Vector();
        for (Iterator iter = focussedProjects.iterator(); iter.hasNext();) {
            IProject project = (IProject) iter.next();
            Vector items = AssistPlugin.getDefault().getChangedEntitiesForProject(project);
            if (items!=null) union.addAll(items);
        }
        return (ModificationItem[]) union.toArray(new ModificationItem[0]);
    }
  
}