/*
 * $Id: PreprocessProjectWizard.java,v 1.5.2.1 2005/04/09 12:43:48 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.assist.ui.wizards;

import java.sql.SQLException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.wizard.Wizard;
import org.softevo.rose.RosePluginException;
import org.softevo.rose.assist.AssistPlugin;
import org.softevo.rose.assist.preprocessing.PreprocessingRosePlugin;
import org.softevo.rose.ccvs.IRepositoryProvider;
import org.softevo.rose.ccvs.RepositoryException;
import org.softevo.rose.ccvs.eclipse.EclipseRepositoryProvider;
import org.softevo.rose.db.DBConnectionProperties;
import org.softevo.rose.db.DatabaseProvider;
import org.softevo.rose.db.DbPlugin;

/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class PreprocessProjectWizard extends Wizard {
    
    /** The wizard page that contains the connections
     *  tree. <BR>
     */
    private ChooseLocationWizardPage choosePage;
    
    /** The project the db connection is set for. <BR>
     */
    private IProject[] projects;
    
    /** Creates  anew wizard instance. <BR>
     *  @param project the project the connection is set for
     */
    public PreprocessProjectWizard(IProject[] projects) {
        super();
        this.projects = projects;
        setWindowTitle("Preprocess Project");
    }
    
    /** Adds the wizard pages. <BR>
     */
    public void addPages() {
        setNeedsProgressMonitor(true);
        if (projects.length>1) {
            choosePage = new ChooseLocationWizardPage("ChooseLocation", "Choose a database location", 
                    AssistPlugin.getDefault().getImageRegistry().getDescriptor("newlocation"),
                    null);
            choosePage.setDescription("Choose a database location for the projects.");
        } else {
            choosePage = new ChooseLocationWizardPage("ChooseLocation", "Choose a database location", 
                    AssistPlugin.getDefault().getImageRegistry().getDescriptor("newlocation"),
                    projects[0]);
            choosePage.setDescription("Choose a database location for the project.");
        }
        addPage(choosePage);
    }
    
    /** Tries to set the database connection from the
     *  choose page in the plugin. <BR>
     */
    public boolean performFinish() {
        DBConnectionProperties properties;
        ErrorDialog errorDialog;
        
        properties = choosePage.getSelectedProperties();
        try {
            for (int i = 0; i < projects.length; i++) {
                DbPlugin.getDefault().setProjectDatabaseData(projects[i], properties);                
            }
            
            //IRepositoryProvider cvs = new EclipseRepositoryProvider(project);
            DatabaseProvider db = DbPlugin.getDefault().getProviderForProject(projects[0]);
            PreprocessingRosePlugin rose = new PreprocessingRosePlugin();
            rose.init(db);
            //rose.initCVS(cvs);
            rose.reset();
            rose.setup();
            rose.run(projects);
            return true;
        } catch (SQLException exception) {
            errorDialog = new ErrorDialog(getShell(), "Failed to connect to database", 
                    "An exception occured during connecting to the database location. " +
                    exception.getClass().getName() + ":" + exception.getMessage(), 
                    Status.CANCEL_STATUS, Status.CANCEL);
            errorDialog.open();
            return false;
        } catch (RosePluginException e) {
            errorDialog = new ErrorDialog(getShell(), "Failed to connect to database", 
                    "An exception occured during connecting to the database location. " +
                    e.getClass().getName() + ":" + e.getMessage(), 
                    Status.CANCEL_STATUS, Status.CANCEL);
            errorDialog.open();
            return false;
        } /*catch (RepositoryException e) {
            errorDialog = new ErrorDialog(getShell(), "Failed to preprocess project", 
                    "An exception occured during preprocessing the project. " +
                    e.getClass().getName() + ":" + e.getMessage(), 
                    Status.CANCEL_STATUS, Status.CANCEL);
            errorDialog.open();
            return false;
        }*/
    }
    
    public boolean canFinish() {
        return choosePage.getSelectedProperties() != null;
    }
}
