/*
 * $Id: AbstractLogOutputListener.java,v 1.1 2004/09/01 15:34:32 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.ccvs.listener;

import java.util.Iterator;

import org.softevo.rose.ccvs.model.BranchItem;
import org.softevo.rose.ccvs.model.LogInfo;
import org.softevo.rose.ccvs.model.RevisionItem;
import org.softevo.rose.ccvs.model.TagItem;

/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public abstract class AbstractLogOutputListener implements ILogOutputListener {
	
    
    /**
     * @see org.softevo.rose.ccvs.listener.ILogOutputListener#doLogInfo(org.softevo.rose.ccvs.model.LogInfo)
     */
    public final void doLogInfo(LogInfo logInfo) {
        // Process log
        doLog(logInfo);
        
        // Process revisions
        for (Iterator iter = logInfo.getRevisions().iterator(); iter.hasNext();) {
            RevisionItem element = (RevisionItem) iter.next();
            doRevision(element);
        }
        
        // Process branches
        for (Iterator iter = logInfo.getBranches().iterator(); iter.hasNext();) {
            BranchItem element = (BranchItem) iter.next();
            doBranch(element);
        }

        // Process tags
        for (Iterator iter = logInfo.getTags().iterator(); iter.hasNext();) {
            TagItem element = (TagItem) iter.next();
            doTag(element);
        }
    }
    
	/**
	 * This method is called for each LogInfo object.
	 * 
	 * @param logInfo  the LogInfo object.
	 * @see LogInfo
	 */
	public abstract void doLog(LogInfo logInfo);
	
	/**
	 * This method is called for each revision.
	 * 
	 * @param revision  the RevisionItem object.
	 * @see RevisionItem
	 */
	public abstract void doRevision(RevisionItem revision);
	
	/**
	 * This method is called for each branch.
	 * 
	 * @param branch  the BranchItem object.
	 * @see BranchItem
	 */
	public abstract void doBranch(BranchItem branch);
	
	/**
	 * This method is called for each tag.
	 * 
	 * @param tag  the TagItem object.
	 * @see TagItem
	 */
	public abstract void doTag(TagItem tag);

}
