/*
 * $Id: BranchItem.java,v 1.2 2004/11/08 13:05:31 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.ccvs.model;

import java.io.Serializable;

/**
 * This class represents a branch of a file in the log information.
 * <P>
 * It consists of a 
 * <UL>
 * <LI> <EM>branchName</EM>, e.g., <TT>JDK_1_5</TT>, the tag given to the branch
 * <LI> <EM>prefixRevision</EM>, e.g., <TT>1.15.2</TT>, used in "branches:"
 * <LI> <EM>internalRevision</EM>, 
 * 			e.g., <TT>1.15.0.2</TT>, used in "symbolic names"
 * <LI> <EM>orginRevision</EM>, e.g., 1.15.
 * </UL>
 * The affected file is directly stored in the LogInfo object.
 * 
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class BranchItem implements Serializable {

    /** The serialization ID */
	private static final long serialVersionUID = 349676243929024061L;
	
	/** The tag given to the branch. */
    private String branchName = null;    
    /** The common prefix of all revisions on the branch. */
    private String prefixRevision = null;
    /** The revision used to identify the branch tag. */
    private String internalRevision = null;
    /** The revision where the branch was created. */
    private String originRevision = null;

    /**
     * Creates a BranchItem objects with the given parameters.
     * 
     * @param branchName  the tag given to the branch.
     * @param prefixRevision  the common prefix of all revisions on the branch.
     * @param internalRevision  the revision that identifies the branch tag.
     * @param originRevision  the revision where the branch originated.
     */
	public BranchItem(String branchName, String prefixRevision, 
	        String internalRevision, String originRevision) {
		this.branchName = branchName;
		this.prefixRevision = prefixRevision;
		this.internalRevision = internalRevision;
		this.originRevision = originRevision;
	}

    /**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "Branch " + branchName 
			+ "  [prefix = " + prefixRevision 
			+ ", origin = " + originRevision + "]";
	}

    /**
     * @return Returns the branchName.
     */
    public String getBranchName() {
        return branchName;
    }
    
    /**
     * @return Returns the internalRevision.
     */
    public String getInternalRevision() {
        return internalRevision;
    }
    
    /**
     * @return Returns the originRevision.
     */
    public String getOriginRevision() {
        return originRevision;
    }
    
    /**
     * @return Returns the prefixRevision.
     */
    public String getPrefixRevision() {
        return prefixRevision;
    }

    /**
     * @param branchName The branchName to set.
     */
    public void setBranchName(String branchName) {
        this.branchName = branchName;
    }

    /**
     * @param internalRevision The internalRevision to set.
     */
    public void setInternalRevision(String internalRevision) {
        this.internalRevision = internalRevision;
    }

    /**
     * @param originRevision The originRevision to set.
     */
    public void setOriginRevision(String originRevision) {
        this.originRevision = originRevision;
    }

    /**
     * @param prefixRevision The prefixRevision to set.
     */
    public void setPrefixRevision(String prefixRevision) {
        this.prefixRevision = prefixRevision;
    }

}
