/*
 * $Id: EntitiesRosePlugin.java,v 1.8 2004/12/21 16:12:17 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.data.entities;

import java.sql.SQLException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.jobs.Job;
import org.softevo.rose.AbstractRoseCVSPlugin;
import org.softevo.rose.RosePluginException;
import org.softevo.rose.data.entities.db.DatabaseOperations;
import org.softevo.rose.data.entities.db.Tables;
import org.softevo.rose.db.DatabaseProvider;

/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class EntitiesRosePlugin extends AbstractRoseCVSPlugin {

    private DatabaseOperations dbOperation = null;

    /**
     * @param db
     * @param cvs
     */
    public EntitiesRosePlugin() {
        super();
    }

    /**
     * @throws RosePluginException
     * @see org.softevo.rose.IRosePlugin#reset()
     */
    public void reset() throws RosePluginException {
        String[] tables = Tables.ALL_TABLES;
        for (int i = 0; i < tables.length; i++) {
            String tableName = dbOperation.getTableName(tables[i]);
            if (dbOperation.existsTable(tableName)) {
                try {
                    dbOperation.dropTable(tableName);
                } catch (SQLException e) {
    	            throw new RosePluginException(e);
                }
            }
        }
    }

    /**
     * @throws RosePluginException
     * @see org.softevo.rose.IRosePlugin#setup()
     */
    public void setup() throws RosePluginException {
        if (!dbOperation.existsTable(dbOperation.getTableName(Tables.TABLE_ENTITIES))) {
	        try {
	            dbOperation.createTableEntities(null);
	        } catch (Exception e) {
	            throw new RosePluginException(e);
	        }
        }
        if (!dbOperation.existsTable(dbOperation.getTableName(Tables.TABLE_CHANGED_ENTITIES))) {
	        try {
	            dbOperation.createTableChangedEntities(null);
	        } catch (Exception e) {
	            throw new RosePluginException(e);
	        }
        }
        try {
            dbOperation.setupStmts();
        } catch (SQLException e) {
            throw new RosePluginException(e);
        }
    }

    public Job createJob(IProject project) {
        EntitiesJob job = new EntitiesJob(dbOperation, cvs);
        return job;
    }

    public void init(DatabaseProvider db) throws RosePluginException {
        super.init(db);
		dbOperation = new DatabaseOperations(db);
		try {
            dbOperation.init();
        } catch (SQLException e) {
            throw new RosePluginException(e);
        }
    }
    
    public void close() throws RosePluginException {
        try {
            dbOperation.close();
        } catch (SQLException e) {
            throw new RosePluginException(e);
        } finally {
            dbOperation = null;
            super.close();
        }
    }

}
