/*
 * $Id: ExtractRosePlugin.java,v 1.5 2004/12/21 16:12:20 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.data.mirror;

import java.sql.SQLException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.jobs.Job;
import org.softevo.rose.AbstractRoseCVSPlugin;
import org.softevo.rose.RosePluginException;
import org.softevo.rose.data.mirror.db.DatabaseOperations;
import org.softevo.rose.data.mirror.db.Tables;
import org.softevo.rose.db.DatabaseProvider;

/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class ExtractRosePlugin extends AbstractRoseCVSPlugin {

    private DatabaseOperations dbOperation = null;

    /**
     * @param db
     * @param cvs
     */
    public ExtractRosePlugin() {
        super();
    }

    /**
     * @throws RosePluginException
     * @see org.softevo.rose.IRosePlugin#reset()
     */
    public void reset() throws RosePluginException {
        String[] tables = Tables.ALL_TABLES;
        for (int i = 0; i < tables.length; i++) {
            String tableName = dbOperation.getTableName(tables[i]);
            if (dbOperation.existsTable(tableName)) {
                try {
                    dbOperation.dropTable(tableName);
                } catch (SQLException e) {
    	            throw new RosePluginException(e);
                }
            }
        }
    }

    /**
     * @throws RosePluginException
     * @see org.softevo.rose.IRosePlugin#setup()
     */
    public void setup() throws RosePluginException {
        if (!dbOperation.existsTable(dbOperation.getTableName(Tables.TABLE_DIRECTORIES))) {
	        try {
	            dbOperation.createTableDirectorys(null);
	        } catch (Exception e) {
	            throw new RosePluginException(e);
	        }
        }
        if (!dbOperation.existsTable(dbOperation.getTableName(Tables.TABLE_FILES))) {
	        try {
	            dbOperation.createTableFiles(null);
	        } catch (Exception e) {
	            throw new RosePluginException(e);
	        }
        }
        if (!dbOperation.existsTable(dbOperation.getTableName(Tables.TABLE_TRANSACTIONS))) {
	        try {
	            dbOperation.createTableTransactions(null);
	        } catch (Exception e) {
	            throw new RosePluginException(e);
	        }
        }
        if (!dbOperation.existsTable(dbOperation.getTableName(Tables.TABLE_REVISIONS))) {
	        try {
	            dbOperation.createTableRevisions(null);
	        } catch (Exception e) {
	            throw new RosePluginException(e);
	        }
        }
        if (!dbOperation.existsTable(dbOperation.getTableName(Tables.TABLE_TAGS))) {
	        try {
	            dbOperation.createTableTags(null);
	        } catch (Exception e) {
	            throw new RosePluginException(e);
	        }
        }
        if (!dbOperation.existsTable(dbOperation.getTableName(Tables.TABLE_BRANCHES))) {
	        try {
	            dbOperation.createTableBranches(null);
	        } catch (Exception e) {
	            throw new RosePluginException(e);
	        }
        }
        try {
            dbOperation.setupStmts();
        } catch (SQLException e) {
            throw new RosePluginException(e);
        }
    }

    public Job createJob(IProject project) {
        ExtractJob job = new ExtractJob(dbOperation, cvs);
		return job;
    }
    
    public void init(DatabaseProvider db) throws RosePluginException {
        super.init(db);
		dbOperation = new DatabaseOperations(db);
		try {
            dbOperation.init();
        } catch (SQLException e) {
            throw new RosePluginException(e);
        }

    }
    
    public void close() throws RosePluginException {
        try {
            dbOperation.close();
        } catch (SQLException e) {
            throw new RosePluginException(e);
        } finally {
            dbOperation = null;
            super.close();
        }
    }

}
