/*
 * $Id: ChooseDBConnectionWizard.java,v 1.6 2005/03/01 12:55:08 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.db.wizards;

import java.sql.SQLException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.internal.Workbench;
import org.softevo.rose.db.DBConnectionProperties;
import org.softevo.rose.db.DbPlugin;

/** A wizard used to choose a database connection
 *  for a project. <BR> 
 */
public class ChooseDBConnectionWizard extends Wizard {

  /** The wizard page that contains the connections
   *  tree. <BR>
   */
  private ChooseDBConnectionWizardPage choosePage;

  /** The project the db connection is set for. <BR>
   */
  private IProject project;
  
  /** Creates  anew wizard instance. <BR>
   *  @param project the project the connection is set for
   */
  public ChooseDBConnectionWizard(IProject project) {
    super();
    this.project = project;
    setWindowTitle("Choose database connection for project");
  }
  
  /** Adds the wizard pages. <BR>
   */
  public void addPages() {
    Workbench workbench;
    
    workbench = Workbench.getInstance(); 
    setNeedsProgressMonitor(true);
    choosePage = new ChooseDBConnectionWizardPage("TestIt", "Choose database connection", 
                                                  workbench.getSharedImages().getImageDescriptor(IDE.SharedImages.IMG_OBJ_PROJECT),
                                                  project);
    addPage(choosePage);
  }
  
  /** Tries to set the database connection from the
   *  choose page in the plugin. <BR>
   */
  public boolean performFinish() {
    DBConnectionProperties properties;
    ErrorDialog errorDialog;
    
    properties = choosePage.getSelectedProperties();
    if (properties == null) {
      errorDialog = new ErrorDialog(getShell(), "No selection", 
                                    "You must choose an existing database connection", 
                                    Status.CANCEL_STATUS, Status.CANCEL);
      errorDialog.open();
      return false;
    }
    try {
        DbPlugin.getDefault().setProjectDatabaseData(project, properties);
        return true;
      } catch (SQLException exception) {
        errorDialog = new ErrorDialog(getShell(), "Exception resetting connection", 
                                      "An exception occured resetting the database connection:" +
                                      exception.getClass().getName() + ":" + exception.getMessage(), 
                                      Status.CANCEL_STATUS, Status.CANCEL);
        errorDialog.open();
        return false;
      }
  }
  
}
