/*
 * $Id: ChooseDBConnectionWizardPage.java,v 1.9 2005/01/13 14:56:47 dallmeier Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.db.wizards;


import org.eclipse.core.resources.IProject;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.widgets.Composite;
import org.softevo.rose.db.DBConnectionProperties;
import org.softevo.rose.db.DatabaseProvider;
import org.softevo.rose.db.DbPlugin;
import org.softevo.rose.db.preferences.DBConnectionsView;

/** A wizard page that displays a connection view. <BR>
 *  The connection view uses another wizard to edit the
 *  connections.
 */
public class ChooseDBConnectionWizardPage extends WizardPage {

  /** The project the database connection is set for. <BR>
   */
  private IProject project;
  
  /** The database connections view. <BR>
   */
  private DBConnectionsView view;

  /** Creates a new wizard page. <BR>
   *  @param pageName the name of the page
   *  @param title the title
   *  @param image an image for the page
   *  @param project the project the connection is set for
   */
  public ChooseDBConnectionWizardPage(String pageName, String title, 
                                      ImageDescriptor image, IProject project) {
    super(pageName, title, image);
    this.project = project;
  }
  
  /** Creates the controls (the connection view). <BR>
   *  @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite parent) {
    DatabaseProvider provider;
    
    view = new DBConnectionsView(parent, project);
    provider = DbPlugin.getDefault().getProviderForProject(project);
    if (provider != null) {
      view.selectPropertiesInTree(provider.getConnectionProperties());
    }
    setControl(parent);
    Dialog.applyDialogFont(parent);
  }
  
  /** Gets the selection from the properties. <BR>
   */
  public DBConnectionProperties getSelectedProperties() {
    return view.getSelection();
  }

}
