/*
 * $Id: StringUtils.java,v 1.3 2005/03/01 12:55:25 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.util;

import java.util.Iterator;
import java.util.List;

/**
 * This class provides helpful string utility functions.
 * 
 * @author zimmerth
 */
public class StringUtils {


	/** The default Tab Size */
	public static final int TAB_SIZE = 4;


	/**
	 * Counts the frequency of a substring in string.
	 * 
	 * @param string  The String in which to search.
	 * @param substring  The search String.
	 * @return int  The number of occurences.
	 */
	public static int countFrequency(String string, String substring) {
		int n = 0;
		for (int pos = string.indexOf(substring);
			 pos > -1;
			 pos = string.indexOf(substring, pos + 1), n++) { /*empty*/ } 
		return n;
	}


	/**
	 * @param string
	 * @return int
	 */
	public static int sizeOfStringWithTabs(String string) {
		return StringUtils.sizeOfStringWithTabs(string, TAB_SIZE);
	}


	/**
	 * 
	 * @param string
	 * @param tabSize
	 * @return int
	 */
	public static int sizeOfStringWithTabs(String string, int tabSize) {
		int size = 0;
		for (int i = 0; i < string.length(); i++) {	
			switch (string.charAt(i)) {
				case '\t' :
					size = ((size / tabSize) + 1) * tabSize;
					break;
				case ' ' :
					size++;
					break;
				default :
					size++;
					break;
			}
		}
		return size;
	}

	/**
	 * @param entityIDs
	 * @param string
	 * @return
	 */
	public static String join(Iterator iter, String delim) {
		StringBuffer buf = new StringBuffer();
		for ( ; iter.hasNext();) {
            Object o = iter.next();
            buf.append(o.toString());
            if (iter.hasNext()) buf.append(delim);
        }
		return buf.toString();
	}


    /**
     * @param array
     * @param string
     * @return
     */
    public static String join(int[] array, String delim) {
    
    	StringBuffer buf = new StringBuffer();
    	
    	// Append the first n-1 entries. If there is only 1, nothing is done.
    	for (int pos = 0; pos < array.length - 1; pos++) {
    		buf.append(array[pos]);
    		buf.append(delim);
    	}
    
    	// Append the last entry without a delimiter.
    	if (array.length > 0) {
    		buf.append(array[array.length - 1]);
    	}
    	
    	return buf.toString();
    }


    /**
     * @param array
     * @param string
     * @return
     */
    public static String join(double[] array, String delim) {
    
    	StringBuffer buf = new StringBuffer();
    	
    	// Append the first n-1 entries. If there is only 1, nothing is done.
    	for (int pos = 0; pos < array.length - 1; pos++) {
    		buf.append(array[pos]);
    		buf.append(delim);
    	}
    
    	// Append the last entry without a delimiter.
    	if (array.length > 0) {
    		buf.append(array[array.length - 1]);
    	}
    	
    	return buf.toString();
    }


    /**
     * @param entityIDs
     * @param string
     * @return
     */
    public static String join(Object[] array, String delim) {
    
    	StringBuffer buf = new StringBuffer();
    	
    	// Append the first n-1 entries. If there is only 1, nothing is done.
    	for (int pos = 0; pos < array.length - 1; pos++) {
    		if (array[pos] != null) {
    			buf.append(array[pos].toString());
    			buf.append(delim);
    		}
    	}
    
    	// Append the last entry without a delimiter.
    	if (array.length > 0) {
    		if (array[array.length - 1] != null) {
    			buf.append(array[array.length - 1]);
    		}
    	}
    	
    	return buf.toString();
    }
	
    
	/**
	 * @param entityIDs
	 * @param string
	 * @return
	 */
	public static String join(List list, String delim) {

		StringBuffer buf = new StringBuffer();

		int pos = 0;
		int length = list.size();
		for (Iterator iter = list.iterator(); iter.hasNext(); pos++) {
			Object element = iter.next();
			buf.append(element.toString());
			if (pos < length - 1) {
				buf.append(delim);			
			}
		}
		
		return buf.toString();
	}
}
